@@ -1,7 +1,12 @@
 import typing as T
-from contextlib import contextmanager
 from pathlib import Path
 
+if T.TYPE_CHECKING:
+    from contextlib import contextmanager
+else:
+    # until https://github.com/agronholm/typeguard/issues/115 is fixed in typeguard 3
+    from snowfakery.backports.typeguard_context_manager_hack import contextmanager
+
 from click.utils import LazyFile
 
 OpenFileLike = T.Union[T.TextIO, LazyFile]
@@ -11,7 +16,7 @@ FileLike = T.Union[OpenFileLike, Path, str]
 @contextmanager
 def open_file_like(
     file_like: T.Optional[FileLike], mode, **kwargs
-) -> T.ContextManager[T.Tuple[str, OpenFileLike]]:
+) -> T.Generator[T.Tuple[T.Optional[Path], T.Optional[OpenFileLike]], None, None]:
     """Look at a file-like or path-like object and open it
 
     Returns a) the path it was given OR a best-guess of the path and
@@ -30,7 +35,7 @@ def open_file_like(
             yield file_like, f
 
     elif hasattr(file_like, "name"):
-        yield file_like.name, file_like
+        yield Path(file_like.name), file_like  # type: ignore
 
     elif hasattr(file_like, "read"):
-        yield None, file_like
+        yield None, T.cast(OpenFileLike, file_like)
